#!/usr/bin/env python3
import pytest

from cozify import cloud, hub, hub_api, config
from cozify.test import debug
from cozify.test.fixtures import *
from cozify.Error import AuthenticationError, APIError, ConnectionError

from mbtest.imposters import Imposter, Predicate, Stub, Response


@pytest.mark.live
def test_hub(live_cloud, live_hub):
    assert live_hub.ping()
    hub_id = live_hub.default()
    assert hub_api.hub(
        hub_id=hub_id,
        host=live_hub.host(hub_id),
        remote=live_hub.remote(hub_id),
        cloud_token=live_cloud.token(),
        hub_token=live_hub.token(hub_id))


@pytest.mark.mbtest
def test_hub_api_timeout(mock_server, tmp_hub):
    imposter = Imposter(
        Stub(Predicate(path="/hub/tz"), Response(body='Europe/Helsinki', wait=6000)))
    with pytest.raises(ConnectionError) as e_info:
        with mock_server(imposter):
            hub_api.tz(host=imposter.host, port=imposter.port, base='', hub_token=tmp_hub.token)



"""Module for handling consistent state storage.

Attributes:
    state_file(str): file path where state storage is kept. By default XDG conventions are used. (Most likely ~/.config/python-cozify/python-cozify.cfg)
    state(configparser.ConfigParser): State object used for in-memory state. By default initialized with _initState.
"""

import configparser
import os
import datetime
from absl import logging


def _initXDG():
    """Initialize config path per XDG basedir-spec and resolve the final location of state file storage.

    Returns:
        str: file path to state file as per XDG spec and current env.
    """

    # per the XDG basedir-spec we adhere to $XDG_CONFIG_HOME if it's set, otherwise assume $HOME/.config
    xdg_config_home = ''
    if 'XDG_CONFIG_HOME' in os.environ:
        xdg_config_home = os.environ['XDG_CONFIG_HOME']
        logging.debug('XDG basedir overriden: {0}'.format(xdg_config_home))
    else:
        xdg_config_home = "%s/.config" % os.path.expanduser('~')

    # XDG base-dir: "If, when attempting to write a file, the destination directory is non-existant an attempt should be made to create it with permission 0700. If the destination directory exists already the permissions should not be changed."
    if not os.path.isdir(xdg_config_home):
        logging.debug('XDG basedir does not exist, creating: {0}'.format(xdg_config_home))
        os.mkdir(xdg_config_home, 0o0700)

    # finally create our own config dir
    config_dir = "%s/%s" % (xdg_config_home, 'python-cozify')
    if not os.path.isdir(config_dir):
        logging.debug('XDG local dir does not exist, creating: {0}'.format(config_dir))
        os.mkdir(config_dir, 0o0700)

    state_file = "%s/python-cozify.cfg" % config_dir
    logging.debug('state_file determined to be: {0}'.format(state_file))
    return state_file


def stateWrite(tmpstate=None):
    """Write current state to file storage.

    Args:
        tmpstate(configparser.ConfigParser): State object to store instead of default state.
    """
    global state_file
    if tmpstate is None:
        global state
        tmpstate = state
    with open(state_file, 'w') as cf:
        tmpstate.write(cf)


def setStatePath(filepath=_initXDG(), copy_current=False):
    """Set state storage path. Useful for example for testing without affecting your normal state. Call with no arguments to reset back to autoconfigured location.

    Args:
        filepath(str): file path to use as new storage location. Defaults to XDG defined path.
        copy_current(bool): Instead of initializing target file, dump previous state into it.
    """
    global state_file
    global state
    state_file = filepath
    if copy_current:
        stateWrite()
    else:
        state = _initState(state_file)


def dump_state():
    """Print out current state file to stdout. Long values are truncated since this is only for visualization.
    """
    for section in state.sections():
        print('[{!s:.10}]'.format(section))
        for option in state.options(section):
            print('  {!s:<13.13} = {!s:>10.100}'.format(option, state[section][option]))


def _iso_now():
    """Helper to return isoformat datetime stamp that's more compatible than the default.
    Once Python 3.6 is more widely popular this becomes redundant since isoformat() will support timespec.

    Returns:
        str: now() in isoformat truncated to full seconds.
    """

    return datetime.datetime.now().isoformat().split(".")[0]


def _initState(state_file):
    """Initialize state on cold start. Any stored state is read in or a new basic state is initialized.

    Args:
        state_file(str): State storage filepath to attempt to read from.
    Returns:
        configparser.ConfigParser: State object.
    """
    # if we can read it, read it in, otherwise create empty file
    state = configparser.ConfigParser(allow_no_value=True)
    try:
        cf = open(state_file, 'r')
    except IOError:
        cf = open(state_file, 'w+')
        os.chmod(state_file, 0o600)  # set to user readwrite only to protect tokens
    else:
        state.read_file(cf)

    # make sure config is in roughly a valid state
    for key in ['Cloud', 'Hubs']:
        if key not in state:
            state[key] = {}
    stateWrite(state)
    return state


state_file = _initXDG()
state = _initState(state_file)



import time

from . import config


# expects Cozify devices type json data
def getMultisensorData(data):  # pragma: no cover
    """Deprecated, will be removed in v0.3
    """
    out = []
    for device in data:
        state = data[device]['state']
        devtype = state['type']

        if devtype == 'STATE_MULTI_SENSOR':
            name = data[device]['name']
            if 'lastSeen' in state:
                timestamp = state['lastSeen']
            else:
                # if no time of measurement is known we must make a reasonable assumption
                # Stored here in milliseconds to match accuracy of what the hub will give you
                timestamp = time.time() * 1000
            if 'temperature' in state:
                temperature = state['temperature']
            else:
                temperature = None
            if 'humidity' in state:
                humidity = state['humidity']
            else:
                humidity = None

            out.append({
                'name': name,
                'time': timestamp,
                'temperature': temperature,
                'humidity': humidity
            })
    return out



#!/usr/bin/env python3
"""Set high log level
"""

from absl import logging
import logging as dlog

# shut up the hyperactive urllib3
urllib3_logger = dlog.getLogger('urllib3')
urllib3_logger.setLevel(dlog.CRITICAL)

logging.set_verbosity(logging.DEBUG)



#!/usr/bin/env python3
import pytest, time

from cozify import hub
from cozify.test import debug
from cozify.test.fixtures import live_hub, tmp_hub, tmp_cloud, online_device
from cozify.Error import APIError

# global timer delay for tests that change device state
delay = 2


@pytest.mark.logic
def test_hub_devices_filter_single(tmp_hub):
    ids, devs = tmp_hub.devices()
    out = hub.devices(hub_id=tmp_hub.id, capabilities=hub.capability.COLOR_LOOP, mock_devices=devs)
    assert all(i in out for i in [ids['lamp_osram'], ids['strip_osram']])
    assert len(out) == 2


@pytest.mark.logic
def test_hub_devices_filter_or(tmp_hub):
    ids, devs = tmp_hub.devices()
    out = hub.devices(
        hub_id=tmp_hub.id,
        and_filter=False,
        capabilities=[hub.capability.TWILIGHT, hub.capability.COLOR_HS],
        mock_devices=devs)
    assert all(i in out for i in [ids['lamp_osram'], ids['strip_osram'], ids['twilight_nexa']])
    assert len(out) == 3


@pytest.mark.logic
def test_hub_devices_filter_and(tmp_hub):
    ids, devs = tmp_hub.devices()
    out = hub.devices(
        hub_id=tmp_hub.id,
        and_filter=True,
        capabilities=[hub.capability.COLOR_HS, hub.capability.COLOR_TEMP],
        mock_devices=devs)
    assert all(i in out for i in [ids['lamp_osram'], ids['strip_osram']])
    assert len(out) == 2


@pytest.mark.logic
def test_hub_device_eligible(tmp_hub):
    ids, devs = tmp_hub.devices()
    assert hub.device_eligible(ids['lamp_osram'], hub.capability.COLOR_TEMP, mock_devices=devs)
    assert not hub.device_eligible(
        ids['twilight_nexa'], hub.capability.COLOR_TEMP, mock_devices=devs)


@pytest.mark.logic
def test_hub_device_implicit_state(tmp_hub):
    ids, devs = tmp_hub.devices()
    state = {}
    hub.device_eligible(
        ids['lamp_osram'], hub.capability.COLOR_TEMP, mock_devices=devs, state=state)
    assert 'temperature' in state
    hub.device_exists(ids['twilight_nexa'], mock_devices=devs, state=state)
    assert 'twilight' in state


@pytest.mark.logic
def test_hub_device_reachable(tmp_hub):
    ids, devs = tmp_hub.devices()
    assert hub.device_reachable(ids['reachable'], mock_devices=devs)
    assert not hub.device_reachable(ids['not-reachable'], mock_devices=devs)
    with pytest.raises(ValueError):
        hub.device_reachable('dead-beef', mock_devices=devs)


@pytest.mark.logic
def test_hub_device_exists(tmp_hub):
    ids, devs = tmp_hub.devices()
    assert hub.device_exists(ids['reachable'], mock_devices=devs)
    assert not hub.device_exists('dead-beef', mock_devices=devs)


@pytest.mark.destructive
@pytest.mark.vcr
def test_hub_device_toggle(live_hub, online_device):
    live_hub.device_toggle(online_device['id'])
    time.sleep(delay)


@pytest.mark.destructive
def test_hub_device_on_off(live_hub, online_device):
    if online_device['state']['isOn']:
        live_hub.device_off(online_device['id'])
        time.sleep(delay)
        live_hub.device_on(online_device['id'])
    else:
        live_hub.device_on(online_device['id'])
    time.sleep(delay)


@pytest.mark.destructive
def test_hub_device_state_replace(live_hub, online_device):
    live_hub.device_on(online_device['id'])

    old_brightness = online_device['state']['brightness']
    if old_brightness > 0.1:
        set_brightness = old_brightness - 0.1
    else:
        set_brightness = 0.5
    online_device['state']['brightness'] = set_brightness
    online_device['state']['isOn'] = True
    live_hub.device_state_replace(online_device['id'], online_device['state'])
    time.sleep(delay)
    devs = live_hub.devices()
    new_brightness = devs[online_device['id']]['state']['brightness']
    new_isOn = devs[online_device['id']]['state']['isOn']

    assert new_brightness != old_brightness, 'brightness did not change, expected {0}'.format(new_brightness)
    assert new_brightness == set_brightness, 'brightness changed unexpectedly, expected {0}'.format(set_brightness)
    assert new_isOn == True



#!/usr/bin/env python3

import os, pytest, tempfile, datetime

from cozify import cloud_api
from cozify.test import debug
from cozify.test.fixtures import *
from cozify.Error import AuthenticationError, APIError, ConnectionError

from mbtest.imposters import Imposter, Predicate, Stub, Response


@pytest.mark.mbtest
def test_cloud_api_mock_lan_ip(mock_server):
    imposter = Imposter(Stub(Predicate(path="/hub/lan_ip"), Response(body='[ "127.0.0.1" ]')))
    with mock_server(imposter):
        assert cloud_api.lan_ip(base=imposter.url)


@pytest.mark.mbtest
def test_cloud_api_timeout(mock_server):
    imposter = Imposter(
        Stub(Predicate(path="/hub/lan_ip"), Response(body='[ "127.0.0.1" ]', wait=6000)))
    with pytest.raises(ConnectionError) as e_info:
        with mock_server(imposter):
            cloud_api.lan_ip(base=imposter.url)


@pytest.mark.mbtest
def test_cloud_api_emaillogin(mock_server, tmp_cloud):
    imposter = Imposter(Stub(Predicate(path="/user/emaillogin"), Response(body=tmp_cloud.token)))
    with mock_server(imposter):
        token = cloud_api.emaillogin(email=tmp_cloud.email, otp='42', base=imposter.url)
        assert isinstance(token, str)



"""Module for handling Cozify Cloud API 1:1 functions

Attributes:
    cloudBase(str): API endpoint including version

"""

import json
import requests

from .Error import APIError, AuthenticationError, ConnectionError

cloudBase = 'https://cloud2.cozify.fi/ui/0.2'


def get(call,
        headers=None,
        base=cloudBase,
        no_headers=False,
        json_output=True,
        raw=False,
        **kwargs):
    """GET method for calling hub API.

    Args:
        call(str): API path to call after the base, needs to include leading /.
        headers(dict): Header dictionary to pass along to the request.
        base(str): Base path to call from API instead of global base. Defaults to cloudBase.
        no_headers(bool): Allow calling without headers or data.
        json_output(bool): Assume API will return json and decode it.
    """
    return _call(method=requests.get,
                 call='{0}{1}'.format(base, call),
                 headers=headers,
                 no_headers=no_headers,
                 json_output=json_output,
                 raw=raw,
                 **kwargs)


def post(call, headers=None, data=None, base=cloudBase, no_headers=False, raw=False, **kwargs):
    """PUT method for calling hub API. For rest of kwargs parameters see get()

    Args:
        call(str): API path to call after apiPath, needs to include leading /.
        headers(dict): Header dictionary to pass along to the request.
        data(dict): Payload dictionary to POST.
        base(str): Base path to call from API instead of global base. Defaults to cloudBase.
        no_headers(bool): Allow calling without headers or data.
    """
    return _call(method=requests.post,
                 call='{0}{1}'.format(base, call),
                 headers=headers,
                 data=data,
                 no_headers=no_headers,
                 raw=raw,
                 **kwargs)


def put(call, headers=None, data=None, base=cloudBase, no_headers=False, raw=False, **kwargs):
    """PUT method for calling hub API. For rest of kwargs parameters see get()

    Args:
        call(str): API path to call after apiPath, needs to include leading /.
        headers(dict): Header dictionary to pass along to the request.
        data(dict): Payload dictionary to PUT.
        base(str): Base path to call from API instead of global base. Defaults to cloudBase.
        no_headers(bool): Allow calling without headers or data.
    """
    return _call(method=requests.put,
                 call='{0}{1}'.format(base, call),
                 headers=headers,
                 no_headers=no_headers,
                 data=data,
                 raw=raw,
                 **kwargs)


def requestlogin(email, **kwargs):  # pragma: no cover
    """Raw Cloud API call, request OTP to be sent to account email address.

    Args:
        email(str): Email address connected to Cozify account.
    """

    payload = {'email': email}
    post('/user/requestlogin', data=payload, **kwargs)


def emaillogin(email, otp, **kwargs):
    """Raw Cloud API call, request cloud token with email address & OTP.

    Args:
        email(str): Email address connected to Cozify account.
        otp(int): One time passcode.

    Returns:
        str: cloud token
    """

    payload = {'email': email, 'password': otp}
    return post('/user/emaillogin', data=payload, json_output=False, **kwargs)


def lan_ip(**kwargs):
    """1:1 implementation of hub/lan_ip

    This call will fail with an APIError if the requesting source address is not the same as that of the hub, i.e. if they're not in the same NAT network.
    The above is based on observation and may only be partially true.

    Returns:
        list: List of Hub ip addresses.
    """
    return list(get('/hub/lan_ip', no_headers=True, **kwargs))


def hubkeys(cloud_token, **kwargs):  # pragma: no cover
    """1:1 implementation of user/hubkeys

    Args:
        cloud_token(str) Cloud remote authentication token.

    Returns:
        dict: Map of hub_id: hub_token pairs.
    """
    headers = {'Authorization': cloud_token}
    return get('/user/hubkeys', headers=headers, **kwargs)


def refreshsession(cloud_token, **kwargs):  # pragma: no cover
    """1:1 implementation of user/refreshsession

    Args:
        cloud_token(str) Cloud remote authentication token.

    Returns:
        str: New cloud remote authentication token. Not automatically stored into state.
    """
    headers = {'Authorization': cloud_token}
    return get('/user/refreshsession', headers=headers, json_output=False, **kwargs)


def remote(apicall, headers, data=None):
    """1:1 implementation of 'hub/remote'

    Args:
        apicall(str): Full API call that would normally go directly to hub, e.g. '/cc/1.6/hub/colors'
        headers(dict): Headers to send with request. Must contain Authorization & X-Hub-Key data.
        data(str): json string to use as payload, changes method to PUT.

    Returns:
        requests.response: Requests response object.
    """

    if data:
        return put('/hub/remote' + apicall, headers=headers, data=data, raw=True)
    else:
        return get('/hub/remote' + apicall, headers=headers, raw=True)


def _call(*,
          call,
          method,
          headers,
          params=None,
          data=None,
          no_headers=False,
          json_output=True,
          raw=False,
          **kwargs):
    """Backend for get & post

    Args:
        call(str): Full API path to call.
        method(function): requests.get|put function to use for call.
        headers(dict): Header dictionary to pass along to the request.
        params(dict): Params dictionary to POST.
        data(dict): Payload dictionary to PUT.
        no_headers(bool): Allow calling without headers, data or args.
        json_output(bool): Assume API will return json and decode it.
        raw(bool): Do no decoding, return requests.response object.
    """
    if not headers and not data and not params and not no_headers:
        raise AttributeError(
            'Asked to do a call to the cloud without valid headers, data or params. This would never work.'
        )

    try:
        if method is requests.put:
            if data:
                response = method(call, headers=headers, data=data, timeout=5)
            else:
                raise AttributeError('PUT call with no data, this would fail!')
        elif params:
            response = method(call, headers=headers, params=params, timeout=5)
        else:
            response = method(call, headers=headers, timeout=5)

    except requests.exceptions.RequestException as e:  # pragma: no cover
        raise ConnectionError(str(e)) from None

    if response.status_code == 200:
        if raw:
            return response
        if json_output:
            return response.json()
        else:
            return response.text

    elif response.status_code == 410:
        raise APIError(
            response.status_code,
            'API version outdated. Update python-cozify. {reason} - {url} - {message}'.format(
                reason=response.reason, url=response.url,
                message=response.text))  # pragma: no cover
    else:
        raise APIError(
            response.status_code, '{reason} - {url} - {message}'.format(reason=response.reason,
                                                                        url=response.url,
                                                                        message=response.text))



#!/usr/bin/env python3

import pytest

import os, tempfile

from cozify import config
from cozify.test import debug
from cozify.test.fixtures import tmp_hub, tmp_cloud


@pytest.mark.logic
def test_config_XDG(tmp_hub):
    assert config._initXDG()


@pytest.mark.logic
def test_config_XDG_env(tmp_hub):
    with tempfile.TemporaryDirectory() as td:
        os.environ["XDG_CONFIG_HOME"] = td
        config.setStatePath(config._initXDG())
        assert td in config.state_file


@pytest.mark.logic
def test_config_XDG_basedir(tmp_hub):
    # using mktemp deliberately to let _initXDG create it
    td = tempfile.mktemp()
    os.environ["XDG_CONFIG_HOME"] = td
    assert config._initXDG()
    assert os.path.isdir(td)
    os.removedirs(td + '/python-cozify')



lamp_ikea = {
    'capabilities': {
        'type':
            'SET',
        'values': [
            'IDENTIFY', 'ALERT', 'ON_OFF', 'CONTROL_LIGHT', 'COLOR_TEMP', 'BRIGHTNESS', 'DEVICE'
        ]
    },
    'groups': ['86397059-1341-4398-8274-3dcef21d0d54'],
    'id': 'd0bd9e1e-9857-4f57-ad53-bc9cbe667c35',
    'manufacturer': 'IKEA of Sweden',
    'model': None,
    'name': 'Table hanger 3',
    'room': ['87658ab7-bc4f-4d03-85a2-eb32ee1d4539'],
    'rwx': 509,
    'state': {
        'brightness': 0.1529,
        'colorMode': 'ct',
        'hue': -1,
        'isOn': True,
        'lastSeen': 1515949335273,
        'maxTemperature': 4000.0,
        'minTemperature': 2202.643171806167,
        'reachable': True,
        'saturation': -1,
        'temperature': 2202.643171806167,
        'transitionMsec': None,
        'type': 'STATE_LIGHT'
    },
    'timestamp': 1515949335281,
    'type': 'LIGHT',
    'zones': []
}

lamp_osram = {
    'capabilities': {
        'type':
            'SET',
        'values': [
            'IDENTIFY', 'ALERT', 'ON_OFF', 'CONTROL_LIGHT', 'TRANSITION', 'COLOR_TEMP',
            'BRIGHTNESS', 'DEVICE', 'COLOR_LOOP', 'COLOR_HS'
        ]
    },
    'description': None,
    'deviceType': None,
    'groups': [],
    'id': 'a371469c-ae3e-11e5-ab7a-68c90bba878f',
    'manufacturer': 'OSRAM',
    'model': 'Classic A60 RGBW',
    'name': 'Dining Täble',
    'room': ['87658ab7-bc4f-4d03-85a2-eb32ee1d4539'],
    'rwx': 509,
    'state': {
        'brightness': 0.4667,
        'colorMode': 'hs',
        'hue': 0.5061454830783556,
        'isOn': False,
        'lastSeen': 1508181980242,
        'maxTemperature': 6622.516556291391,
        'minTemperature': 2000.0,
        'reachable': False,
        'saturation': 1,
        'temperature': -1,
        'transitionMsec': None,
        'type': 'STATE_LIGHT'
    },
    'timestamp': 1515949468373,
    'type': 'LIGHT',
    'zones': []
}

strip_osram = {
    'capabilities': {
        'type':
            'SET',
        'values': [
            'IDENTIFY', 'ALERT', 'ON_OFF', 'CONTROL_LIGHT', 'TRANSITION', 'COLOR_TEMP',
            'BRIGHTNESS', 'DEVICE', 'COLOR_LOOP', 'COLOR_HS'
        ]
    },
    'groups': ['bc5eb203-1b98-491b-9184-6a855b344a32'],
    'id': '4bec213d-8319-4d02-ac2d-6cf34d80ae73',
    'manufacturer': 'OSRAM',
    'model': 'Flex RGBW',
    'name': 'JP Bookshelf',
    'room': ['be69e1df-b552-42cb-b9fb-eecf8c7087c7'],
    'rwx': 509,
    'state': {
        'brightness': 0.5297,
        'colorMode': 'hs',
        'hue': 0.2617993877991494,
        'isOn': True,
        'lastSeen': 1515949638592,
        'maxTemperature': 6622.516556291391,
        'minTemperature': 1501.5015015015015,
        'reachable': True,
        'saturation': 1,
        'temperature': -1,
        'transitionMsec': None,
        'type': 'STATE_LIGHT'
    },
    'timestamp': 1515949638596,
    'type': 'LIGHT',
    'zones': []
}

twilight_nexa = {
    'capabilities': {
        'type': 'SET',
        'values': ['DEVICE', 'TWILIGHT']
    },
    'description': None,
    'deviceType': None,
    'groups': [],
    'id': 'cd9bd0da-f1d5-11e5-8834-68c90bba878f',
    'manufacturer': 'Nexa',
    'model': 'Twilight Sensor',
    'name': 'Nexa Twilight 1',
    'room': [],
    'rwx': 509,
    'state': {
        'lastSeen': 1515845023652,
        'reachable': True,
        'twilight': True,
        'twilightStart': 1515845022577,
        'twilightStop': 1515832900640,
        'type': 'STATE_TWILIGHT'
    },
    'timestamp': 1515845023656,
    'type': 'TWILIGHT',
    'zones': []
}

plafond_osram = {
    'capabilities': {
        'type':
            'SET',
        'values': [
            'IDENTIFY', 'ALERT', 'ON_OFF', 'CONTROL_LIGHT', 'TRANSITION', 'COLOR_TEMP',
            'BRIGHTNESS', 'DEVICE'
        ]
    },
    'groups': [],
    'id': '720b5285-06a3-4069-81e1-519d5b45048d',
    'manufacturer': 'OSRAM',
    'model': 'Surface Light TW',
    'name': 'Lower Stairway',
    'room': ['87658ab7-bc4f-4d03-85a2-eb32ee1d4539'],
    'rwx': 509,
    'state': {
        'brightness': 0,
        'colorMode': 'ct',
        'hue': -1,
        'isOn': False,
        'lastSeen': 1515951870541,
        'maxTemperature': 6535.9477124183,
        'minTemperature': 2702.7027027027025,
        'reachable': True,
        'saturation': -1,
        'temperature': 2702.7027027027025,
        'transitionMsec': None,
        'type': 'STATE_LIGHT'
    },
    'timestamp': 1515951870545,
    'type': 'LIGHT',
    'zones': []
}

state_clean = {
    'brightness': None,
    'colorMode': None,
    'hue': None,
    'isOn': None,
    'lastSeen': None,
    'maxTemperature': None,
    'minTemperature': None,
    'reachable': None,
    'saturation': None,
    'temperature': None,
    'transitionMsec': None,
    'type': 'STATE_LIGHT'
}

devices = {
    lamp_ikea['id']: lamp_ikea,
    lamp_osram['id']: lamp_osram,
    strip_osram['id']: strip_osram,
    plafond_osram['id']: plafond_osram,
    twilight_nexa['id']: twilight_nexa
}

device_ids = {
    'lamp_ikea': lamp_ikea['id'],
    'lamp_osram': lamp_osram['id'],
    'strip_osram': strip_osram['id'],
    'plafond_osram': plafond_osram['id'],
    'twilight_nexa': twilight_nexa['id'],
    'reachable': plafond_osram['id'],
    'not-reachable': lamp_osram['id']
}

states = {'dirty': lamp_ikea['state'], 'clean': state_clean}
